@@ -62,13 +62,20 @@
 //! extern crate cpython;
 //!
 //! use cpython::{PythonObject, Python};
-//! 
+//! use cpython::ObjectProtocol; //for call method
+//!
 //! fn main() {
-//!     let gil_guard = Python::acquire_gil();
-//!     let py = gil_guard.python();
+//!     let gil = Python::acquire_gil();
+//!     let py = gil.python();
+//!
 //!     let sys = py.import("sys").unwrap();
-//!     let version = sys.get("version").unwrap().extract::<String>().unwrap();
-//!     println!("Hello Python {}", version);
+//!     let version: String = sys.get("version").unwrap().extract().unwrap();
+//!
+//!     let os = py.import("os").unwrap();
+//!     let getenv = os.get("getenv").unwrap();
+//!     let user: String = getenv.call(("USER",), None).unwrap().extract().unwrap();
+//!
+//!     println!("Hello {}, I'm Python {}", user, version);
 //! }
 //! ```
 
@@ -88,7 +95,7 @@ pub use err::{PyErr, PyResult};
 pub use objects::*;
 pub use python::{Python, PythonObject, PythonObjectWithCheckedDowncast, PythonObjectWithTypeObject};
 pub use pythonrun::{GILGuard, GILProtected, prepare_freethreaded_python};
-pub use conversion::{FromPyObject, ToPyObject};
+pub use conversion::{ExtractPyObject, ToPyObject};
 pub use objectprotocol::{ObjectProtocol};
 pub use rustobject::{PyRustType, PyRustObject};
 pub use rustobject::typebuilder::PyRustTypeBuilder;
@@ -119,6 +126,7 @@ mod conversion;
 mod objects;
 mod objectprotocol;
 mod pythonrun;
+pub mod argparse;
 mod function;
 mod rustobject;
 
@@ -161,15 +169,15 @@ pub mod _detail {
 /// #![feature(plugin)]
 /// #![plugin(interpolate_idents)]
 /// #[macro_use] extern crate cpython;
-/// use cpython::{Python, PyResult, PyObject, PyTuple};
+/// use cpython::{Python, PyResult, PyObject};
 ///
 /// py_module_initializer!(example, |py, m| {
 ///     try!(m.add("__doc__", "Module documentation string"));
-///     try!(m.add("run", py_fn!(run)));
+///     try!(m.add("run", py_fn!(run())));
 ///     Ok(())
 /// });
-/// 
-/// fn run<'p>(py: Python<'p>, args: &PyTuple<'p>) -> PyResult<'p, PyObject<'p>> {
+///
+/// fn run<'p>(py: Python<'p>) -> PyResult<'p, PyObject<'p>> {
 ///     println!("Rust says: Hello Python!");
 ///     Ok(py.None())
 /// }
@@ -187,7 +195,7 @@ pub mod _detail {
 /// >>> example.run()
 /// Rust says: Hello Python!
 /// ```
-/// 
+///
 #[macro_export]
 #[cfg(feature="python27-sys")]
 macro_rules! py_module_initializer {
