@@ -19,8 +19,9 @@
 use ffi;
 use python::{Python, ToPythonPointer, PythonObject};
 use conversion::ToPyObject;
-use objects::PyObject;
+use objects::{PyObject, PyList};
 use err::{self, PyResult, PyErr};
+use std::{mem, collections, hash, cmp};
 
 /// Represents a Python `dict`.
 pub struct PyDict<'p>(PyObject<'p>);
@@ -77,7 +78,7 @@ impl <'p> PyDict<'p> {
     pub fn get_item<K>(&self, key: K) -> Option<PyObject<'p>> where K: ToPyObject<'p> {
         let py = self.python();
         key.with_borrowed_ptr(py, |key| unsafe {
-            PyObject::from_borrowed_ptr_opt(py, 
+            PyObject::from_borrowed_ptr_opt(py,
                 ffi::PyDict_GetItem(self.as_ptr(), key))
         })
     }
@@ -102,5 +103,48 @@ impl <'p> PyDict<'p> {
                 ffi::PyDict_DelItem(self.as_ptr(), key))
         })
     }
+
+    // List of dict items.
+    // This is equivalent to the python expression `list(dict.items())`.
+    pub fn items_list(&self) -> PyList<'p> {
+        let py = self.python();
+        unsafe {
+            err::cast_from_owned_ptr_or_panic(py, ffi::PyDict_Items(self.as_ptr()))
+        }
+    }
+
+    /// Returns the list of (key,value) pairs in this dictionary.
+    pub fn items(&self) -> Vec<(PyObject<'p>, PyObject<'p>)> {
+        // Note that we don't provide an iterator because
+        // PyDict_Next() is unsafe to use when the dictionary might be changed
+        // by other python code.
+        let py = self.python();
+        let mut vec = Vec::with_capacity(self.len());
+        unsafe {
+            let mut pos = 0;
+            let mut key: *mut ffi::PyObject = mem::uninitialized();
+            let mut value: *mut ffi::PyObject = mem::uninitialized();
+            while ffi::PyDict_Next(self.as_ptr(), &mut pos, &mut key, &mut value) != 0 {
+                vec.push((PyObject::from_borrowed_ptr(py, key),
+                          PyObject::from_borrowed_ptr(py, value)));
+            }
+        }
+        vec
+    }
+}
+
+impl <'p, K, V> ToPyObject<'p> for collections::HashMap<K, V>
+    where K: hash::Hash+cmp::Eq+ToPyObject<'p>,
+          V: ToPyObject<'p>
+{
+    type ObjectType = PyDict<'p>;
+
+    fn to_py_object(&self, py: Python<'p>) -> PyDict<'p> {
+        let dict = PyDict::new(py);
+        for (key, value) in self.iter() {
+            dict.set_item(key, value).unwrap();
+        };
+        dict
+    }
 }
 
