@@ -39,6 +39,35 @@ pub use self::num::PyInt;
 pub use self::num::PyLong as PyInt;
 pub use self::num::{PyLong, PyFloat};
 
+/// Identity conversion: allows using existing `PyObject` instances where
+/// `T: ToPyObject` is expected.
+macro_rules! pyobject_to_pyobject(
+    ($name: ident) => (
+        impl <'p, 's> ::conversion::ToPyObject<'p> for $name<'s> {
+            type ObjectType = $name<'p>;
+
+            #[inline]
+            fn to_py_object(&self, py: ::python::Python<'p>) -> $name<'p> {
+                self.clone().into_py_object(py)
+            }
+
+            #[inline]
+            fn into_py_object(self, _py: ::python::Python<'p>) -> $name<'p> {
+                // Transmute the lifetime.
+                // This is safe, because both lifetime variables represent the same lifetime:
+                // that of the python GIL acquisition.
+                unsafe { ::std::mem::transmute(self) }
+            }
+
+            #[inline]
+            fn with_borrowed_ptr<F, R>(&self, _py: ::python::Python<'p>, f: F) -> R
+              where F: FnOnce(*mut ffi::PyObject) -> R {
+                f(self.as_ptr())
+            }
+        }
+    )
+);
+
 macro_rules! pyobject_newtype(
     ($name: ident) => (
         /// Clone returns another reference to the Python object,
@@ -50,12 +79,14 @@ macro_rules! pyobject_newtype(
             }
         }
 
+        pyobject_to_pyobject!($name);
+
         impl <'p> ::python::PythonObject<'p> for $name<'p> {
             #[inline]
             fn as_object(&self) -> &::objects::object::PyObject<'p> {
                 &self.0
             }
-            
+
             #[inline]
             fn into_object(self) -> ::objects::object::PyObject<'p> {
                 self.0
@@ -67,18 +98,23 @@ macro_rules! pyobject_newtype(
             unsafe fn unchecked_downcast_from(obj: ::objects::object::PyObject<'p>) -> Self {
                 $name(obj)
             }
-            
+
             /// Unchecked downcast from PyObject to Self.
             /// Undefined behavior if the input object does not have the expected type.
             #[inline]
             unsafe fn unchecked_downcast_borrow_from<'a>(obj: &'a ::objects::object::PyObject<'p>) -> &'a Self {
                 ::std::mem::transmute(obj)
             }
+
+            #[inline]
+            fn python(&self) -> ::python::Python<'p> {
+                self.0.python()
+            }
         }
     );
     ($name: ident, $checkfunction: ident) => (
         pyobject_newtype!($name);
-        
+
         impl <'p> ::python::PythonObjectWithCheckedDowncast<'p> for $name<'p> {
             #[inline]
             fn downcast_from(obj : ::objects::object::PyObject<'p>) -> Result<$name<'p>, ::python::PythonObjectDowncastError<'p>> {
@@ -90,7 +126,7 @@ macro_rules! pyobject_newtype(
                     }
                 }
             }
-            
+
             #[inline]
             fn downcast_borrow_from<'a>(obj : &'a ::objects::object::PyObject<'p>) -> Result<&'a $name<'p>, ::python::PythonObjectDowncastError<'p>> {
                 unsafe {
@@ -105,7 +141,7 @@ macro_rules! pyobject_newtype(
     );
     ($name: ident, $checkfunction: ident, $typeobject: ident) => (
         pyobject_newtype!($name, $checkfunction);
-        
+
         impl <'p> ::python::PythonObjectWithTypeObject<'p> for $name<'p> {
             #[inline]
             fn type_object(py: ::python::Python<'p>) -> ::objects::typeobject::PyType<'p> {
@@ -115,6 +151,28 @@ macro_rules! pyobject_newtype(
     );
 );
 
+macro_rules! extract(
+    ($obj:ident to $t:ty => $body: block) => {
+        impl <'python, 'source, 'prepared>
+            ::conversion::ExtractPyObject<'python, 'source, 'prepared>
+            for $t
+            where 'python: 'source
+        {
+
+            type Prepared = &'source PyObject<'python>;
+
+            #[inline]
+            fn prepare_extract(obj: &'source PyObject<'python>) -> PyResult<'python, Self::Prepared> {
+                Ok(obj)
+            }
+
+            fn extract(&$obj: &'prepared &'source PyObject<'python>) -> PyResult<'python, Self> {
+                $body
+            }
+        }
+    }
+);
+
 mod object;
 mod typeobject;
 mod module;
@@ -131,3 +189,4 @@ pub mod exc;
 #[cfg(feature="python27-sys")]
 pub mod oldstyle;
 
+mod tests;
