@@ -69,6 +69,8 @@ impl <'p> Clone for PyObject<'p> {
     }
 }
 
+pyobject_to_pyobject!(PyObject);
+
 impl <'p> PythonObject<'p> for PyObject<'p> {
     #[inline]
     fn as_object<'a>(&'a self) -> &'a PyObject<'p> {
@@ -241,8 +243,10 @@ impl <'p> PyObject<'p> {
     /// Extracts some type from the Python object.
     /// This is a wrapper function around `FromPyObject::from_py_object()`.
     #[inline]
-    pub fn extract<T>(&self) -> Result<T, PyErr<'p>> where T: ::conversion::FromPyObject<'p> {
-        ::conversion::FromPyObject::from_py_object(self)
+    pub fn extract<'s, T>(&'s self) -> Result<T, PyErr<'p>>
+      where T: for<'prep> ::conversion::ExtractPyObject<'p, 's, 'prep> {
+        let prepared = try!(<T as ::conversion::ExtractPyObject>::prepare_extract(self));
+        <T as ::conversion::ExtractPyObject>::extract(&prepared)
     }
 }
 
